// ---------------------------------------------------------------------------
// Praktikum, Aufgabe D2:
// Zweipunktregler: LEDs an PB5/PD6 ein, wenn Spannung an PC0 ber 2 Volt
// steigt, bzw. wieder aus, wenn Spannung an PC0 unter 1,5 Volt abfllt.
// Beim Auslesen des AD-Wandlers werden jeweils 8 Messwerte gemittelt.
// ---------------------------------------------------------------------------

#define F_CPU 16000000UL
#include <avr/io.h>
#include <stdint.h>

#define INTERNAL_LED PB5  // LED an PB5 (auf der Arduino-Platine)
#define EXTERNAL_LED PD6  // LED an PC6 (extern angeschlossen)
#define LIMIT_LO (uint16_t)(1.5 * 1023.0 / 5.0)  // Untere Schwelle
#define LIMIT_HI (uint16_t)(2.0 * 1023.0 / 5.0)  // Obere Schwelle

// ---------------------------------------------------------------------------
// Digitalausgang fr LED initialisieren, AD-Wandler initialisieren
// ---------------------------------------------------------------------------
void init_ports_and_adc(void)
{
    // Ausgnge mit Leuchtdioden
    DDRB = _BV(INTERNAL_LED);
    DDRD = _BV(EXTERNAL_LED);

    // Referenz fr AD-Wandler = VCC = 5V (Datenblatt S. 317)
    ADMUX = _BV(REFS0);
    
    // AD-Wandler einschalten, Prescaler = 128 (Datenblatt S. 320)
    ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
}

// ---------------------------------------------------------------------------
// Analogwert von Eingang ADC0 (Anschluss PC0) einlesen:
// Rckgabewert 0...1023 entspricht Spannung von 0...5 Volt
// ---------------------------------------------------------------------------
uint16_t read_adc0(void)
{
    // ADSC-Bit setzen, um Wandlung zu starten, das ADSC-Bit wird nach
    // dem Ende der Wandlung automatisch wieder gelscht. Der eingelesene
    // 10-Bit-Wert steht anschlieend im Register ADC.
    ADCSRA |= _BV(ADSC);
    while(ADCSRA & _BV(ADSC)) { }  // Auf das Ende der Wandlung warten...
    return ADC;
}

// ---------------------------------------------------------------------------
// 8 Analogwerte von Eingang ADC0 (Anschluss PC0) einlesen und mitteln:
// Rckgabewert 0...1023 entspricht Spannung von 0...5 Volt
// ---------------------------------------------------------------------------
uint16_t read_adc0_mean(void)
{
    uint16_t sum = 0;
    uint8_t i;
    for(i = 0; i < 8; i++) sum += read_adc0();
    return sum / 8;
}

// ---------------------------------------------------------------------------
// Hauptprogramm
// ---------------------------------------------------------------------------
int main(void)
{
    init_ports_and_adc();
    while(1)
    {
        uint16_t adc = read_adc0_mean();
        if(adc > LIMIT_HI)
        {
            PORTB |= _BV(INTERNAL_LED);
            PORTD |= _BV(EXTERNAL_LED);
        }
        if(adc < LIMIT_LO)
        {
            PORTB &= ~_BV(INTERNAL_LED);
            PORTD &= ~_BV(EXTERNAL_LED);
        }
    }
}
